@@ -138,6 +138,13 @@ namespace internal
         return std::back_inserter(ys);
     }
 
+    // Avoid self-assignment.
+    template <typename T>
+    void assign(T& x, T&& y) {
+        if (&x != &y)
+            x = std::move(y);
+    }
+
     template <typename T, std::size_t N>
     struct array_back_insert_iterator : public std::back_insert_iterator<std::array<T, N>>
     {
@@ -166,7 +173,7 @@ namespace internal
         array_back_insert_iterator<T, N>& operator=(T&& x)
         {
             assert(pos_ < N);
-            (*arr_ptr_)[pos_] = std::move(x);
+            assign((*arr_ptr_)[pos_], std::move(x));
             ++pos_;
             return *this;
         }
